#ifndef MathsHelpH
#define MathsHelpH

#include <math.h>

//trig functions in degrees
double __fastcall cosD(double degrees);
double __fastcall sinD(double degrees);
double __fastcall tanD(double degrees);
double __fastcall acosD(double x);
double __fastcall asinD(double x);
double __fastcall atanD(double x);

//matrix methods and vector methods
typedef struct  {
    double aa, ba, ca;
    double ab, bb, cb;
    double ac, bc, cc;
    } matrix;

typedef struct  {
	double x, y;
	} vector2D;

typedef struct  {
	double x, y, z;
	} vector3D;

typedef struct  {
    double x, y, z;
    double parameter;
    bool found;
    } intersection3D;

typedef struct {
	//defines a plane with equation Ax + By + Cz + D = 0;
	double A, B, C, D;
	} plane;

//2D functions
//Length of Points[] MUST be 4. Works for swept angles of 90 degrees or less. Returns false if angle negative or greater than 90.
//start point on Points[0], end point is Points[3] and the other points are control points.
bool __fastcall BezierArc2D(vector2D Points[], vector2D CentrePoint, double radius, double startangle, double endangle);
vector2D __fastcall RotateVector2D(vector2D vector1, vector2D centrepoint, double degrees);
vector2D __fastcall TranslateVector2D(vector2D vector1, vector2D translation);
vector2D __fastcall UnitVector(vector2D vector1);
vector2D __fastcall ScaleVector(vector2D vector1, double scalar);
double __fastcall ScalarProduct(vector2D vector1, vector2D vector2);
double __fastcall DistanceBetweenPoints(vector2D vector1, vector2D vector2);
vector2D __fastcall AddVector(vector2D vector1, vector2D vector2);
vector2D __fastcall SubtractVector(vector2D vector1, vector2D vector2);

//3D functions
matrix __fastcall MatrixMultiply(matrix Matrix1, matrix Matrix2);
vector3D __fastcall MatrixMultiply(matrix Matrix1, vector3D Vector1);
vector3D __fastcall UnitVector(vector3D vector1);
vector3D __fastcall VectorProduct(vector3D vector1, vector3D vector2);
vector3D __fastcall ScaleVector(vector3D vector1, double scalar);
double __fastcall ScalarProduct(vector3D vector1, vector3D vector2);
double __fastcall DistanceBetweenPoints(vector3D vector1, vector3D vector2);
vector3D __fastcall AddVector(vector3D vector1, vector3D vector2);
vector3D __fastcall SubtractVector(vector3D vector1, vector3D vector2);
vector3D __fastcall PlaneNormal(vector3D vector1, vector3D vector2, vector3D vector3);
plane __fastcall Plane(vector3D vector1, vector3D vector2, vector3D vector3);
bool __fastcall PointOnLine(vector3D LineStart,vector3D LineFinish,vector3D Point1);
intersection3D __fastcall Intersection(plane Plane, vector3D LineStart, vector3D LineFinish);
vector3D __fastcall CubicBezier(vector3D P1, vector3D P2, vector3D P3, vector3D P4, double t);


//Various Miscellaneous functions
int __fastcall Heavyside(double value, double zeroaccuracy);
//note, decimalplaces can be negative if say 1024.03 needs to be rounded to the nearest 1000 (ie decimalplaces = -3)
double __fastcall ROUND(double value, int decimalplaces);
//rounds to nearest. So, RoundToNearest(17.43,0.25) returns 17.5 as that's the nearest 0.25 divisible number to 17.43
float __fastcall RoundToNearest_Float(float value, float RoundingValue);
double __fastcall RoundToNearest_Double(double value, double RoundingValue);
//factorial limit is 17.
int __fastcall Factorial(int value);
int __fastcall Binomial(int entry, int order);

//The Max & min macros seem to have disappeared from stlib in RAD Studio 10.1, so use these
int __fastcall min(int value1, int value2);
int __fastcall mini(int value1, int value2);      //sometimes min() for an int causes a name ambiguity on compiling, so use this
int __fastcall max(int value1, int value2);
float __fastcall min(float value1, float value2);
float __fastcall max(float value1, float value2);
double __fastcall min(double value1, double value2);
double __fastcall max(double value1, double value2);

#endif

