//---------------------------------------------------------------------------
#ifndef PDFUnitH
#define PDFUnitH

#include <stdlib.h>
#include <stdio.h>
//may have to change the next 2 includes dependent on BCB version
#include <Vcl.Forms.hpp>
#include <Vcl.Graphics.hpp>
#include "PageSizeHelp.h"

//structures etc unique to the PDF unit
//These enums are no-longer unique & have been moved to PageSizeHelp, definitions retained here for easy reference
//                         0           1             2      3      4      5      6      7        8               9         10       11       12       13         14             15     16     17     18     19     20     21     22        23        24        25       26        27                      28          29              30              31                  32              33             34           35                    36                         37                       38                     39                    40                  41               42                     43
//enum TPageSize        {psA0      , psA1        , psA2 , psA3 , psA4 , psA5 , psA6 , psLegal, psJuniorLegal , psANSIE , psANSID, psANSIC, psANSIB, psLetter , psHalfLetter , psB0 , psB1 , psB2 , psB3 , psB4 , psB5 , psB6 , psArchE , psArchD , psArchC , psArchB, psArchA , psCustom } ;
//enum TPaperDirection  {pdPortrait, pdLandscape };

typedef struct {
    int startoffset, generation;
    bool f;
	} objectlist;

typedef struct {
    int objectnumber, generation;
    } resource;

typedef struct {
    AnsiString Name;
    //font data is unused for image or annots resources.
    AnsiString FontName;
    bool bold, italic, underlined, strikeout;
    int objectnumber, generation;
	} resourcelist;

typedef struct {
	AnsiString Name;
	int Size;
	TColor Color;
	} PDFFont;

typedef struct {
	TBrushStyle Style;
	//int Style;
	TColor Color;
	} PDFBrush;

typedef struct {
    int pagenumber, objectnumber, generation;
    int parentobjectnumber, parentgeneration;
    int contentsobjectnumber, contentsgeneration;
    bool used;
    bool resourceImageB, resourceImageC, resourceText;
    DynamicArray <resource> FontResource;
    DynamicArray <resource> ImageResource;
    DynamicArray <resource> AnnotsResource;
    } pagelist;

typedef struct {
    double X,Y;
    } coordinates;
//---------------------------------------------------------------------------
class TPDFObject : public TObject
{
private:
    //methods
    virtual void __fastcall AfterConstruction(void);
    virtual void __fastcall BeforeDestruction(void);

    void __fastcall AddObject(int objectnumber, int generation, bool f);

    void __fastcall FlushPageBuffer(void);
    void __fastcall WriteCatalog(void);
    void __fastcall WriteOutlines(void);
    void __fastcall WritePageContents(void);
    void __fastcall WritePage(void);
    void __fastcall WritePages(void);
    void __fastcall WriteProducer(void);
    void __fastcall WriteLine(AnsiString text);
    void __fastcall WritePageLine(AnsiString text);
    void __fastcall WriteFontResource(bool Type1Font);
    void __fastcall CRLF(void);
    AnsiString __fastcall ClippingArea(double TopLeftX, double TopLeftY, double BottomRightX, double BottomRightY);
    AnsiString __fastcall GetPenColour(void);
    AnsiString __fastcall GetBrushColour(void);
    AnsiString __fastcall GetTextColour(void);
    AnsiString __fastcall SanitiseText(AnsiString text);
	AnsiString __fastcall Octal(char a);
	void __fastcall MakeEnoughPages(int PageNumber);

    //Variables
    TComponent* PDFOwner;
    int streampointer;
    DynamicArray <objectlist> ObjectList;
    DynamicArray <pagelist> PageList;
    DynamicArray <resourcelist> ImageResourceList;
    DynamicArray <resourcelist> FontResourceList;
    FILE *output;
    int nextobjectnumber;
    int nextpagenumber;
    int currentpageparent;
    TStringList *PageBuffer;

    //variables for public properties
    TPageSize fPageSize;
    double fPageWidth;
    double fPageHeight;
    double fLeftMargin;
    double fTopMargin;
    double fRightMargin;
    double fBottomMargin;
    double fPenWidth;
    double fPenPosX;
    double fPenPosY;
    TColor fPenColour;
    TColor fBrushColour;
	TPageDirection fPageOrientation;
    AnsiString fFileName;
    AnsiString fTitle;
    AnsiString fAuthor;
	AnsiString fCreator;
	AnsiString fKeywords;
	AnsiString fSubject;
    bool fLaunchOnExit;

    //methods for public properties
    void __fastcall SetPageSize(TPageSize Size);
    void __fastcall SetPageWidth(double Width);
    void __fastcall SetPageHeight(double Height);
    void __fastcall SetPageOrientation(/*TPrinterOrientation*/TPageDirection Orientation);
    void __fastcall SetFileName(AnsiString Name);
    void __fastcall SetPenColour(TColor Colour);
    void __fastcall SetPenWidth(double Width);
    void __fastcall SetBrushColour(TColor Colour);

    //variable for private properties
    bool fPageOpen;
    bool fTextOpen;
    bool fFileOpen;

    //private properties
    __property bool FileOpen = {read = fFileOpen, write = fFileOpen};
    __property bool PageOpen = {read = fPageOpen, write = fPageOpen};
    __property bool TextOpen = {read = fTextOpen, write = fTextOpen};
public:
    //All dimensions relating to the page are real world millimetres, from the top left corner of the page.
    //properties
    __property TComponent *Owner = {read = PDFOwner};
    __property TPageSize PageSize = {read = fPageSize, write = SetPageSize};
    __property double PageWidth = {read = fPageWidth, write = SetPageWidth};
    __property double PageHeight = {read = fPageHeight, write = SetPageHeight};
    __property double LeftMargin = {read = fLeftMargin, write = fLeftMargin};
    __property double TopMargin = {read = fTopMargin, write = fTopMargin};
    __property double RightMargin = {read = fRightMargin, write = fRightMargin};
    __property double BottomMargin = {read = fBottomMargin, write = fBottomMargin};
	__property TPageDirection PageOrientation = {read = PageOrientation, write = SetPageOrientation};
    __property AnsiString FileName = {read = fFileName, write = SetFileName};
    __property AnsiString Title = {read = fTitle, write = fTitle};
    __property AnsiString Author = {read = fAuthor, write = fAuthor};
	__property AnsiString Creator = {read = fCreator, write = fCreator};
	__property AnsiString Keywords = {read = fKeywords, write = fKeywords};
	__property AnsiString Subject = {read = fSubject, write = fSubject};
	__property bool LaunchOnExit = {read = fLaunchOnExit, write = fLaunchOnExit};

	//variables
	PDFFont Font;
	TFontStyles FontStyle;
    TPen *Pen;
	PDFBrush Brush;
    __property double PenWidth = {read = fPenWidth, write = SetPenWidth};
    __property double PenPosX = {read = fPenPosX, write = fPenPosX};
    __property double PenPosY = {read = fPenPosY, write = fPenPosY};
    __property TColor PenColour = {read = fPenColour, write = SetPenColour};
	__property TColor BrushColour = {read = fBrushColour, write = SetBrushColour};

	//methods
	//document commands
    //'BeginDocument()' returns true if file opened OK. If not, owner must kill the PDFObject.
    bool __fastcall BeginDocument(AnsiString Name);
    void __fastcall NewPage(void);
    void __fastcall EndDocument(void);

	//graphics commands
	void __fastcall AngleArc(double CentreX, double CentreY, double Radius, double StartAngle, double SweepAngle);
	void __fastcall Arc(double X1, double Y1, double X2, double Y2, double X3, double Y3, double X4, double Y4);
    //returns the resource object number so the graphic can be re-used if desired 
    int __fastcall Draw(Graphics::TBitmap *Bitmap, double TopLeftX, double TopLeftY, double BottomRightX, double BottomRightY);
	void __fastcall Chord(double X1, double Y1, double X2, double Y2, double X3, double Y3, double X4, double Y4);
	void __fastcall Ellipse(double TopLeftX, double TopLeftY, double BottomRightX, double BottomRightY);
	void __fastcall LineTo(double X, double Y);
	void __fastcall MoveTo(double X, double Y);
	void __fastcall Pie(double X1, double Y1, double X2, double Y2, double X3, double Y3, double X4, double Y4);
	void __fastcall PolyBezier(coordinates Points[], int pointsize);
    void __fastcall Polygon(coordinates Points[], int pointsize);
	void __fastcall Polyline(coordinates Points[], int pointsize);
	void __fastcall PolyLine(coordinates Points[], int pointsize);
	void __fastcall Rectangle(double TopLeftX, double TopLeftY, double BottomRightX, double BottomRightY);
    void __fastcall RoundRect(double TopLeftX, double TopLeftY, double BottomRightX, double BottomRightY, double EllipseWidth, double EllipseHeight);

    //text commands
	double __fastcall TextHeight(void); //returns the 'official' height of the text font.
    double __fastcall TextHeight(AnsiString text); //returns the actual heigh of the text
	double __fastcall TextWidth(AnsiString text);  //returns the actual width of the text
	void __fastcall BeginText(void);    //debatable whether this will ever need to be called
	void __fastcall TextOut(AnsiString text, double TopLeftX, double TopLeftY); //Obsolete: I got it wrong way round first, and then had done too much coding in the caller program this way round to delete this version
	void __fastcall TextOut(double TopLeftX, double TopLeftY, AnsiString text);
	void __fastcall EndText(void);      //debatable whether this will ever need to be called

    //Hyperlinking
    void __fastcall URL(AnsiString DisplayText, AnsiString Destination, double TopLeftX, double TopLeftY);
    void __fastcall PageLink(AnsiString DisplayText, int DestinationPage, double TopLeftX, double TopLeftY);
	//Note: HotSpot seems to be unreliable, preference not to use.
    void __fastcall HotSpot(int DestinationPage, double TopLeftX, double TopLeftY, double BottomRightX, double BottomRightY);

    //Image Resource Commands
    int __fastcall AddResourceImage(Graphics::TBitmap *Bitmap);
	void __fastcall UseResourceImage(int ObjectNumber, double TopLeftX, double TopLeftY, double BottomRightX, double BottomRightY);

	//Comment
    void __fastcall Comment(AnsiString CommentText);

    //instantiation
    __fastcall TPDFObject(TComponent* Owner);
};
//---------------------------------------------------------------------------
#endif
