//---------------------------------------------------------------------------

#pragma hdrstop

#include "PageSizeHelp.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

/*
When finding the page sizes, I consulted the ISO page specs, Wikipedia and any 
other sources I could find. I also measured the paper in my printer, cut 
sheets for the plotter and everything else I could get my hands on.

The size tolerances in the ISO spec gave the smallest variation I could find.
Actual paper sheets were quite badly under tolerance, up to 3mm smaller than
the minimum specified anywhere, giving a 10mm tolerance between min & max sizes. 
I have tried to cater for this variation in the sizes, particularly in the 
'WhatPageIsThis' function.

*/
pagesize __fastcall GetPageSize(TPageSize Size)
{
	pagesize Page;
	Page.PaperDirection = pdPortrait;
	Page.dpi = 600;
    switch (Size) {
        case psA0:     {
						Page.PageWidth = 841.0;
						Page.PageHeight = 1189.0;
						Page.PageSize = Size;
						}
						break;
		case psA1:     {
						Page.PageWidth = 594.0;
						Page.PageHeight = 841.0;
						Page.PageSize = Size;
						}
						break;
		case psA2:     {
						Page.PageWidth = 420.0;
						Page.PageHeight = 594.0;
						Page.PageSize = Size;
						}
						break;
		case psA3:     {
						Page.PageWidth = 297.0;
						Page.PageHeight = 420.0;
						Page.PageSize = Size;
						}
						break;
		case psA4:     {
						Page.PageWidth = 210.0;
						Page.PageHeight = 297.0;
						Page.PageSize = Size;
						}
						break;
		case psA5:     {
						Page.PageWidth = 148.0;
						Page.PageHeight = 210.0;
						Page.PageSize = Size;
                        }
						break;
        case psA6:     {
                        Page.PageWidth = 105.0;
						Page.PageHeight = 148.0;
                        Page.PageSize = Size;
                        }
                        break;
		case psANSIE: {
                        Page.PageWidth = 864.0;
                        Page.PageHeight = 1118.0;
                        Page.PageSize = Size;
                        }
						break;
        case psANSID: {
                        Page.PageWidth = 559.0;
                        Page.PageHeight = 864.0;
                        Page.PageSize = Size;
						}
                        break;
        case psANSIC: {
                        Page.PageWidth = 432.0;
                        Page.PageHeight = 559.0;
						Page.PageSize = Size;
                        }
                        break;
        case psANSIB: {
                        Page.PageWidth = 279.0;
						Page.PageHeight = 432.0;
                        Page.PageSize = Size;
                        }
                        break;
        case psLetter: {
						Page.PageWidth = 215.9;
                        Page.PageHeight = 279.4;
                        Page.PageSize = Size;
						}
						break;
        case psHalfLetter: {
						Page.PageWidth = 140.0;
                        Page.PageHeight = 216.0;
						Page.PageSize = Size;
                        }
                        break;
		case psLegal:  {
                        Page.PageWidth = 215.9;
                        Page.PageHeight = 355.6;
						Page.PageSize = Size;
                        }
                        break;
		case psJuniorLegal: {
                        Page.PageWidth = 127.0;
                        Page.PageHeight = 203.0;
						Page.PageSize = Size;
                        }
                        break;
		case psB0:     {
                        Page.PageWidth = 1000.0;
                        Page.PageHeight = 1414.0;
						Page.PageSize = Size;
                        }
                        break;
		case psB1:     {
                        Page.PageWidth = 707.0;
                        Page.PageHeight = 1000.0;
						Page.PageSize = Size;
                        }
                        break;
		case psB2:     {
                        Page.PageWidth = 500.0;
                        Page.PageHeight = 707.0;
						Page.PageSize = Size;
                        }
						break;
        case psB3:     {
						Page.PageWidth = 353.0;
                        Page.PageHeight = 500.0;
						Page.PageSize = Size;
                        }
						break;
        case psB4:     {
                        Page.PageWidth = 250.0;
						Page.PageHeight = 353.0;
                        Page.PageSize = Size;
                        }
                        break;
		case psB5:     {
                        Page.PageWidth = 176.0;
                        Page.PageHeight = 250.0;
                        Page.PageSize = Size;
						}
                        break;
        case psB6:     {
                        Page.PageWidth = 125.0;
						Page.PageHeight = 176.0;
                        Page.PageSize = Size;
                        }
                        break;
		case psArchE:  {
                        Page.PageWidth = 914.0;
                        Page.PageHeight = 1219.0;
                        Page.PageSize = Size;
						}
                        break;
        case psArchD:  {
                        Page.PageWidth = 610.0;
						Page.PageHeight = 914.0;
                        Page.PageSize = Size;
                        }
                        break;
		case psArchC:  {
                        Page.PageWidth = 457.0;
                        Page.PageHeight = 610.0;
                        Page.PageSize = Size;
						}
                        break;
        case psArchB:  {
                        Page.PageWidth = 305.0;
						Page.PageHeight = 457.0;
                        Page.PageSize = Size;
                        }
                        break;
		case psArchA:  {
						Page.PageWidth = 229.0;
						Page.PageHeight = 305.0;
						Page.PageSize = Size;
						}
                        break;
		default:        {
						Page.PageSize = psCustom;
						Page.PageWidth = 210.0;
                        Page.PageHeight = 297.0;
					   }
						break;
		}
	return Page;
}
//---------------------------------------------------------------------------

pagesize __fastcall GetPageSize(TPageSize Size, TPageDirection Direction)
{
	pagesize Page = GetPageSize(Size);

	if (Direction == pdLandscape) {
		Page.PaperDirection = Direction;
		double swap = Page.PageWidth;
		Page.PageWidth = Page.PageHeight;
        Page.PageHeight = swap;
		}

	return Page;

}
//---------------------------------------------------------------------------

pagesize __fastcall GetLandscapePage(TPageSize Size)
{
	pagesize Page = GetPageSize(Size,pdLandscape);
	return Page;
}
//---------------------------------------------------------------------------

pagesize __fastcall GetPortraitPage(TPageSize Size)
{
	pagesize Page = GetPageSize(Size);
	return Page;
}
//---------------------------------------------------------------------------

pagesize __fastcall WhatPageIsThis(double Width, double Height)
{
	pagesize Page;
	bool found = false;
    Page.dpi = 600;

	if (!found && Width > 1178.0 && Width < 1192.0) {
		//may be A0
		if (Height > 830.0 && Height < 844.0) {
			//It's A0 landscape
			Page.PageSize = psA0;
			Page.PageWidth = 1189.0;
			Page.PageHeight = 841.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 830.0 && Width < 844.0) {
		//may be A0 portrait or A1 landscape
		if (Height > 1178.0 && Height < 1192.0) {
			//It's A0 portrait
			Page.PageSize = psA0;
			Page.PageWidth = 841.0;
			Page.PageHeight = 1189.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 583.0 && Height < 596.0) {
			//It's A1 landscape
			Page.PageSize = psA1;
			Page.PageWidth = 841.0;
			Page.PageHeight = 594.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 583.0 && Width < 596.0) {
		//may be A1 portrait or A2 landscape
		if (Height > 830.0 && Height < 844.0) {
			//It's A1 portrait
			Page.PageSize = psA1;
			Page.PageWidth = 594.0;
			Page.PageHeight = 841.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 409.0 && Height < 422.0) {
			//It's A2 landscape
			Page.PageSize = psA2;
			Page.PageWidth = 594.0;
			Page.PageHeight = 420.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 409.0 && Width < 422.0) {
		//may be A2 portrait or A3 landscape
		if (Height > 583.0 && Height < 596.0) {
			//It's A2 portrait
			Page.PageSize = psA2;
			Page.PageWidth = 420.0;
			Page.PageHeight = 594.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 286.0 && Height < 299.0) {
			//It's A3 landscape
			Page.PageSize = psA3;
			Page.PageWidth = 420.0;
			Page.PageHeight = 297.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 286.0 && Width < 299.0) {
		//may be A3 portrait or A4 landscape
		if (Height > 409.0 && Height < 422.0) {
			//It's A3 portrait
			Page.PageSize = psA3;
			Page.PageWidth = 297.0;
			Page.PageHeight = 420.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 199.0 && Height < 212.0) {
			//It's A4 landscape
			Page.PageSize = psA4;
			Page.PageWidth = 297.0;
			Page.PageHeight = 210.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 199.0 && Width < 212.0) {
		//may be A4 portrait or A5 landscape
		if (Height > 286.0 && Height < 299.0) {
			//It's A4 portrait
			Page.PageSize = psA4;
			Page.PageWidth = 210.0;
			Page.PageHeight = 297.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 139.5 && Height < 149.5) {
			//It's A5 landscape
			Page.PageSize = psA5;
			Page.PageWidth = 210.0;
			Page.PageHeight = 148.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 139.5 && Width < 149.5) {
		//may be A5 portrait or A6 landscape
		if (Height > 199.0 && Height < 212.0) {
			//It's A5 portrait
			Page.PageSize = psA5;
			Page.PageWidth = 148.0;
			Page.PageHeight = 210.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 96.5 && Height < 106.5) {
			//It's A6 landscape
			Page.PageSize = psA6;
			Page.PageWidth = 148.0;
			Page.PageHeight = 105.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 96.5 && Width < 106.5) {
		//may be A6 portrait
		if (Height > 139.5 && Height < 149.5) {
			//It's A6 portrait
			Page.PageSize = psA6;
			Page.PageWidth = 105.0;
			Page.PageHeight = 148.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		}

	if (!found && Width > 1115.0 && Width < 1121.0) {
		//may be AnsiE
		if (Height > 861.0 && Height < 867.0) {
			//It's AnsiE landscape
			Page.PageSize = psANSIE;
			Page.PageWidth = 1118.0;
			Page.PageHeight = 864.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 861.0 && Width < 867.0) {
		//may be AnsiE portrait or AnsiD landscape
		if (Height > 1115.0 && Height < 1121.0) {
			//It's AnsiE portrait
			Page.PageSize = psANSIE;
			Page.PageWidth = 864.0;
			Page.PageHeight = 1118.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 557.0 && Height < 561.0) {
			//It's AnsiD landscape
			Page.PageSize = psANSID;
			Page.PageWidth = 864.0;
			Page.PageHeight = 559.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 557.0 && Width < 561.0) {
		//may be AnsiD portrait or AnsiC landscape
		if (Height > 861.0 && Height < 867.0) {
			//It's AnsiD portrait
			Page.PageSize = psANSID;
			Page.PageWidth = 559.0;
			Page.PageHeight = 864.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 430.0 && Height < 434.0) {
			//It's AnsiD landscape
			Page.PageSize = psANSIC;
			Page.PageWidth = 559.0;
			Page.PageHeight = 432.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 430.0 && Width < 434.0) {
		//may be AnsiC portrait or AnsiB landscape
		if (Height > 557.0 && Height < 561.0) {
			//It's AnsiC portrait
			Page.PageSize = psANSIC;
			Page.PageWidth = 432.0;
			Page.PageHeight = 559.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 277.0 && Height < 281.0) {
			//It's AnsiD landscape
			Page.PageSize = psANSIB;
			Page.PageWidth = 432.0;
			Page.PageHeight = 279.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 268.0 && Width < 281.4) {
		//may be AnsiB portrait or Letter landscape
		if (Height > 430.0 && Height < 434.0) {
			//It's AnsiB portrait
			Page.PageSize = psANSIC;
			Page.PageWidth = 279.4;
			Page.PageHeight = 432.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 205.9 && Height < 217.9) {
			//It's Letter landscape
			Page.PageSize = psLetter;
			Page.PageWidth = 279.4;
			Page.PageHeight = 215.9;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 205.9 && Width < 217.9) {
		//may be Legal portrait, Letter portrait or HalfLetter landscape
		if (Height > 345.6 && Height < 357.6) {
			//It's Legal portrait
			Page.PageSize = psLegal;
			Page.PageWidth = 215.9;
			Page.PageHeight = 279.4;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (Height > 268.0 && Height < 281.4) {
			//It's Letter portrait
			Page.PageSize = psLetter;
			Page.PageWidth = 215.9;
			Page.PageHeight = 279.4;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 138.5 && Height < 141.5) {
			//It's HalfLetter landscape
			Page.PageSize = psHalfLetter;
			Page.PageWidth = 215.9;
			Page.PageHeight = 140.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 138.5 && Width < 141.5) {
		//may be Half Letter portrait
		if (Height > 213.9 && Height < 217.9) {
			//It's HalfLetter portrait
			Page.PageSize = psHalfLetter;
			Page.PageWidth = 140.0;
			Page.PageHeight = 215.9;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		}

	if (!found && Width > 345.6 && Width < 357.6) {
		//may be Legal landscape
		if (Height > 205.9 && Height < 217.9) {
			//It's Legal landscape
			Page.PageSize = psLegal;
			Page.PageWidth = 355.6;
			Page.PageHeight = 215.9;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 201.0 && Width < 205.0) {
		//may be Junior Legal landscape
		if (Height > 125.0 && Height < 129.0) {
			//It's Junior Legal landscape
			Page.PageSize = psJuniorLegal;
			Page.PageWidth = 203.0;
			Page.PageHeight = 127.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 125.0 && Width < 129.0) {
		//may be Junior Legal portrait
		if (Height > 125.0 && Height < 129.0) {
			//It's Junior Legal portrait
			Page.PageSize = psJuniorLegal;
			Page.PageWidth = 127.0;
			Page.PageHeight = 203.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		}

	if (!found && Width > 1403.0 && Width < 1417.0) {
		//may be B0
		if (Height > 989.0 && Height < 1003.0) {
			//It's B0 landscape
			Page.PageSize = psB0;
			Page.PageWidth = 1414.0;
			Page.PageHeight = 1000.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 989.0 && Width < 1003.0) {
		//may be B0 portrait or B1 landscape
		if (Height > 1403.0 && Height < 1417.0) {
			//It's B0 portrait
			Page.PageSize = psB0;
			Page.PageWidth = 1000.0;
			Page.PageHeight = 1414.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 696.0 && Height < 710.0) {
			//It's B1 landscape
			Page.PageSize = psB1;
			Page.PageWidth = 1000.0;
			Page.PageHeight = 707.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 696.0 && Width < 710.0) {
		//may be B1 portrait or B2 landscape
		if (Height > 989.0 && Height < 1003.0) {
			//It's B1 portrait
			Page.PageSize = psB1;
			Page.PageWidth = 707.0;
			Page.PageHeight = 1000.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 489.0 && Height < 502.0) {
			//It's B2 landscape
			Page.PageSize = psB2;
			Page.PageWidth = 707.0;
			Page.PageHeight = 500.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 489.0 && Width < 502.0) {
		//may be B2 portrait or B3 landscape
		if (Height > 696.0 && Height < 710.0) {
			//It's B2 portrait
			Page.PageSize = psB2;
			Page.PageWidth = 500.0;
			Page.PageHeight = 707.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 342.0 && Height < 355.0) {
			//It's B3 landscape
			Page.PageSize = psB3;
			Page.PageWidth = 500.0;
			Page.PageHeight = 353.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 342.0 && Width < 355.0) {
		//may be B3 portrait or B4 landscape
		if (Height > 489.0 && Height < 502.0) {
			//It's B3 portrait
			Page.PageSize = psB3;
			Page.PageWidth = 353.0;
			Page.PageHeight = 500.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 239.0 && Height < 252.0) {
			//It's B4 landscape
			Page.PageSize = psB4;
			Page.PageWidth = 353.0;
			Page.PageHeight = 250.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 239.0 && Width < 252.0) {
		//may be B4 portrait or B5 landscape
		if (Height > 342.0 && Height < 355.0) {
			//It's B4 portrait
			Page.PageSize = psB4;
			Page.PageWidth = 250.0;
			Page.PageHeight = 353.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 166.0 && Height < 178.0) {
			//It's B5 landscape
			Page.PageSize = psB5;
			Page.PageWidth = 250.0;
			Page.PageHeight = 176.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 166.0 && Width < 178.0) {
		//may be B5 portrait or B6 landscape
		if (Height > 239.0 && Height < 252.0) {
			//It's B5 portrait
			Page.PageSize = psB5;
			Page.PageWidth = 176.0;
			Page.PageHeight = 250.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 123.5 && Height < 126.5) {
			//It's B6 landscape
			Page.PageSize = psB6;
			Page.PageWidth = 176.0;
			Page.PageHeight = 125.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 123.5 && Width < 126.5) {
		//may be B6 portrait
		if (Height > 174.0 && Height < 178.0) {
			//It's B6 portrait
			Page.PageSize = psB6;
			Page.PageWidth = 125.0;
			Page.PageHeight = 176.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		}

	if (!found && Width > 1208.0 && Width < 1221.0) {
		//may be ArchE Landscape
		if (Height > 904.0 && Height < 917.0) {
			//It's ArchE landscape
			Page.PageSize = psArchE;
			Page.PageWidth = 1219.0;
			Page.PageHeight = 914.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 904.0 && Width < 917.0) {
		//may be ArchE portrait or ArchD landscape
		if (Height > 1208.0 && Height < 1221.0) {
			//It's ArchE portrait
			Page.PageSize = psArchE;
			Page.PageWidth = 914.0;
			Page.PageHeight = 1219.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 599.9 && Height < 613.0) {
			//It's ArchD landscape
			Page.PageSize = psArchD;
			Page.PageWidth = 914;
			Page.PageHeight = 610.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 599.9 && Width < 613.0) {
		//may be ArchD portrait or ArchC landscape
		if (Height > 903.0 && Height < 917.0) {
			//It's ArchD portrait
			Page.PageSize = psArchD;
			Page.PageWidth = 610.0;
			Page.PageHeight = 914.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 446.0 && Height < 459.0) {
			//It's ArchC landscape
			Page.PageSize = psArchC;
			Page.PageWidth = 610.0;
			Page.PageHeight = 457.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 446.0 && Width < 459.0) {
		//may be ArchC portrait or ArchB landscape
		if (Height > 599.9 && Height < 613.0) {
			//It's ArchC portrait
			Page.PageSize = psArchC;
			Page.PageWidth = 457.0;
			Page.PageHeight = 610.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 294.0 && Height < 307.0) {
			//It's ArchB landscape
			Page.PageSize = psArchB;
			Page.PageWidth = 457.0;
			Page.PageHeight = 305.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 294.0 && Width < 307.0) {
		//may be ArchB portrait or ArchA landscape
		if (Height > 446.0 && Height < 459.0) {
			//It's ArchB portrait
			Page.PageSize = psArchB;
			Page.PageWidth = 305.0;
			Page.PageHeight = 457.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		if (!found && Height > 218.0 && Height < 231.0) {
			//It's ArchA landscape
			Page.PageSize = psArchA;
			Page.PageWidth = 305.0;
			Page.PageHeight = 229.0;
			Page.PaperDirection = pdLandscape;
			found = true;
			}
		}

	if (!found && Width > 218.0 && Width < 231.0) {
		//may be ArchA portrait
		if (Height > 294.0 && Height < 307.0) {
			//It's ArchA portrait
			Page.PageSize = psArchA;
			Page.PageWidth = 229.0;
			Page.PageHeight = 305.0;
			Page.PaperDirection = pdPortrait;
			found = true;
			}
		}

	if (!found) {
		Page.PageSize = psCustom;
		Page.PageWidth = Width;
		Page.PageHeight = Height;
		if (Height > Width) {
			Page.PaperDirection = pdPortrait;
			} else {
			Page.PaperDirection = pdLandscape;
            }
		}
    return Page;
}
//---------------------------------------------------------------------------

AnsiString __fastcall PageName(TPageSize PageSize)
{
	//gives the name of the page specified
	AnsiString value;

	switch ((int)(PageSize)) {
		case psA0:  {
					value = "A0";
					}
					break;
		case psA1:  {
					value = "A1";
					}
					break;
		case psA2:  {
					value = "A2";
					}
					break;
		case psA3:  {
					value = "A3";
					}
					break;
		case psA4:  {
					value = "A4";
					}
					break;
		case psA5:  {
					value = "A5";
					}
					break;
		case psA6:  {
					value = "A6";
					}
					break;
		case psLegal:  {
					value = "Legal";
					}
					break;
		case psJuniorLegal:  {
					value = "Junior Legal";
					}
					break;
		case psANSIE:  {
					value = "Ansi E";
					}
					break;
		case psANSID:  {
					value = "Ansi D";
					}
					break;
		case psANSIC:  {
					value = "Ansi C";
					}
					break;
		case psANSIB:  {
					value = "Ansi B/ Tabloid";
					}
					break;
		case psLetter:  {
					value = "Ansi A/ Letter";
					}
					break;
		case psHalfLetter:  {
					value = "Half Letter";
					}
					break;
		case psB0:  {
					value = "B0";
					}
					break;
		case psB1:  {
					value = "B1";
					}
					break;
		case psB2:  {
					value = "B2";
					}
					break;
		case psB3:  {
					value = "B3";
					}
					break;
		case psB4:  {
					value = "B4";
					}
					break;
		case psB5:  {
					value = "B5";
					}
					break;
		case psB6:  {
					value = "B6";
					}
					break;
		case psArchE:  {
					value = "Arch E";
					}
					break;
		case psArchD:  {
					value = "Arch D";
					}
					break;
		case psArchC:  {
					value = "Arch C";
					}
					break;
		case psArchB:  {
					value = "Arch B";
					}
					break;
		case psArchA:  {
					value = "Arch A";
					}
					break;
		default:    {
					value = "Custom Size";
					}
		}

	return value;
}
//---------------------------------------------------------------------------

